@@ -10,7 +10,11 @@ use base64::Engine;
 use flate2::write::GzEncoder;
 use flate2::Compression;
 use fs_err as fs;
+#[cfg(unix)]
+use fs_err::os::unix::fs::OpenOptionsExt;
 use fs_err::File;
+#[cfg(unix)]
+use fs_err::OpenOptions;
 use ignore::overrides::Override;
 use ignore::WalkBuilder;
 use indexmap::IndexMap;
@@ -20,17 +24,15 @@ use std::collections::{HashMap, HashSet};
 use std::env;
 use std::ffi::OsStr;
 use std::fmt::Write as _;
-#[cfg(unix)]
-use std::fs::OpenOptions;
 use std::io;
 use std::io::{Read, Write};
 #[cfg(unix)]
-use std::os::unix::fs::{OpenOptionsExt, PermissionsExt};
+use std::os::unix::fs::PermissionsExt;
 use std::path::{Path, PathBuf};
 use std::process::{Command, Output};
 use std::str;
 use tempfile::{tempdir, TempDir};
-use tracing::debug;
+use tracing::{debug, instrument};
 use zip::{self, DateTime, ZipWriter};
 
 /// Allows writing the module to a wheel or add it directly to the virtualenv
@@ -706,6 +708,7 @@ fn handle_cffi_call_result(
 
 /// Copies the shared library into the module, which is the only extra file needed with bindings
 #[allow(clippy::too_many_arguments)]
+#[instrument(skip_all)]
 pub fn write_bindings_module(
     writer: &mut impl ModuleWriter,
     project_layout: &ProjectLayout,
@@ -1209,8 +1212,12 @@ pub fn write_python_part(
                         continue;
                     }
                 }
+                #[cfg(unix)]
+                let mode = absolute.metadata()?.permissions().mode();
+                #[cfg(not(unix))]
+                let mode = 0o644;
                 writer
-                    .add_file(relative, &absolute)
+                    .add_file_with_permissions(relative, &absolute, mode)
                     .context(format!("File to add file from {}", absolute.display()))?;
             }
         }
